@@ -125,6 +125,8 @@
 #endif
 
 #include <string.h>
+#include <math.h>
+
 #include <gobject/gvaluecollector.h>
 
 #include "clutter-animator.h"
@@ -1217,14 +1219,9 @@ clutter_animator_set (ClutterAnimator *animator,
           break;
         }
 
-#if GLIB_CHECK_VERSION (2, 23, 2)
       G_VALUE_COLLECT_INIT (&value, G_PARAM_SPEC_VALUE_TYPE (pspec),
                             args, 0,
                             &error);
-#else
-      g_value_init (&value, G_PARAM_SPEC_VALUE_TYPE (pspec));
-      G_VALUE_COLLECT (&value, args, 0, &error);
-#endif /* GLIB_CHECK_VERSION (2, 23, 2) */
 
       if (error)
         {
@@ -1429,7 +1426,6 @@ again:
           ClutterAnimatorKey *prev_key = NULL;
           key->is_inert = is_inert;
 
-          clutter_animator_key_free (key);
 
           /* FIXME: non performant since we reiterate the list many times */
 
@@ -1449,6 +1445,8 @@ again:
                   next_key->ease_in = key->ease_in;
                 }
             }
+
+          clutter_animator_key_free (key);
           priv->score = g_list_remove (priv->score, key);
           goto again;
         }
@@ -1767,7 +1765,6 @@ static void
 clutter_animator_class_init (ClutterAnimatorClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
-  GParamSpec *pspec;
 
   g_type_class_add_private (klass, sizeof (ClutterAnimatorPrivate));
 
@@ -1784,14 +1781,13 @@ clutter_animator_class_init (ClutterAnimatorClass *klass)
    *
    * Since: 1.2
    */
-  pspec = g_param_spec_uint ("duration",
-                             P_("Duration"),
-                             P_("The duration of the animation"),
-                             0, G_MAXUINT,
-                             2000,
-                             CLUTTER_PARAM_READWRITE);
-  obj_props[PROP_DURATION] = pspec;
-  g_object_class_install_property (gobject_class, PROP_DURATION, pspec);
+  obj_props[PROP_DURATION] =
+    g_param_spec_uint ("duration",
+                       P_("Duration"),
+                       P_("The duration of the animation"),
+                       0, G_MAXUINT,
+                       2000,
+                       CLUTTER_PARAM_READWRITE);
 
   /**
    * ClutterAnimator:timeline:
@@ -1801,13 +1797,16 @@ clutter_animator_class_init (ClutterAnimatorClass *klass)
    *
    * Since: 1.2
    */
-  pspec = g_param_spec_object ("timeline",
-                               P_("Timeline"),
-                               P_("The timeline of the animation"),
-                               CLUTTER_TYPE_TIMELINE,
-                               CLUTTER_PARAM_READWRITE);
-  obj_props[PROP_TIMELINE] = pspec;
-  g_object_class_install_property (gobject_class, PROP_TIMELINE, pspec);
+  obj_props[PROP_TIMELINE] =
+    g_param_spec_object ("timeline",
+                         P_("Timeline"),
+                         P_("The timeline of the animation"),
+                         CLUTTER_TYPE_TIMELINE,
+                         CLUTTER_PARAM_READWRITE);
+
+  _clutter_object_class_install_properties (gobject_class,
+                                            PROP_LAST,
+                                            obj_props);
 }
 
 static void
@@ -1989,19 +1988,9 @@ clutter_animator_property_set_interpolation (ClutterAnimator     *animator,
     }
 }
 
-GType
-clutter_animator_key_get_type (void)
-{
-  static GType our_type = 0;
-
-  if (!our_type)
-    our_type = g_boxed_type_register_static (I_("ClutterAnimatorKey"),
-                                             clutter_animator_key_copy,
-                                             clutter_animator_key_free);
-
-  return our_type;
-}
-
+G_DEFINE_BOXED_TYPE (ClutterAnimatorKey, clutter_animator_key,
+                     clutter_animator_key_copy,
+                     clutter_animator_key_free);
 
 /**
  * clutter_animator_key_get_object:
