@@ -334,7 +334,15 @@ clutter_get_motion_events_enabled (void)
   return context->motion_events_per_actor;
 }
 
-guint _clutter_pix_to_id (guchar pixel[4]);
+static inline ClutterActor *
+_clutter_actor_get_by_id (guint32 id)
+{
+  ClutterMainContext *context = _clutter_context_get_default ();
+
+  g_assert (context->id_pool != NULL);
+
+  return clutter_id_pool_lookup (context->id_pool, id);
+}
 
 void
 _clutter_id_to_color (guint id, ClutterColor *col)
@@ -599,7 +607,7 @@ _clutter_do_pick (ClutterStage   *stage,
         }
 
       id = _clutter_pixel_to_id (pixel);
-      actor = clutter_get_actor_by_gid (id);
+      actor = _clutter_actor_get_by_id (id);
       goto result;
     }
 
@@ -691,7 +699,7 @@ _clutter_do_pick (ClutterStage   *stage,
     }
 
   id = _clutter_pixel_to_id (pixel);
-  actor = clutter_get_actor_by_gid (id);
+  actor = _clutter_actor_get_by_id (id);
 
 result:
 
@@ -2555,7 +2563,6 @@ _clutter_process_event (ClutterEvent *event)
   context->current_event = NULL;
 }
 
-
 /**
  * clutter_get_actor_by_gid:
  * @id: a #ClutterActor ID.
@@ -2570,13 +2577,7 @@ _clutter_process_event (ClutterEvent *event)
 ClutterActor *
 clutter_get_actor_by_gid (guint32 id)
 {
-  ClutterMainContext *context;
-
-  context = _clutter_context_get_default ();
-
-  g_return_val_if_fail (context != NULL, NULL);
-
-  return CLUTTER_ACTOR (clutter_id_pool_lookup (context->id_pool, id));
+  return _clutter_actor_get_by_id (id);
 }
 
 void
@@ -3267,3 +3268,47 @@ clutter_get_default_text_direction (void)
 {
   return clutter_text_direction;
 }
+
+/*< private >
+ * clutter_clear_events_queue:
+ *
+ * Clears the events queue stored in the main context.
+ */
+void
+_clutter_clear_events_queue (void)
+{
+  ClutterMainContext *context = _clutter_context_get_default ();
+
+  if (context->events_queue != NULL)
+    {
+      g_queue_foreach (context->events_queue,
+                       (GFunc) clutter_event_free,
+                       NULL);
+      g_queue_free (context->events_queue);
+      context->events_queue = NULL;
+    }
+}
+
+void
+_clutter_clear_events_queue_for_stage (ClutterStage *stage)
+{
+  ClutterMainContext *context = _clutter_context_get_default ();
+  GList *l, *next;
+
+  if (context->events_queue == NULL)
+    return;
+
+  /* Remove any pending events for this stage from the event queue */
+  for (l = context->events_queue->head; l; l = next)
+    {
+      ClutterEvent *event = l->data;
+
+      next = l->next;
+
+      if (event->any.stage == stage)
+        {
+          g_queue_delete_link (context->events_queue, l);
+          clutter_event_free (event);
+        }
+    }
+}
