@@ -113,7 +113,7 @@ struct _ClutterStage
  * @fullscreen: handler for the #ClutterStage::fullscreen signal
  * @unfullscreen: handler for the #ClutterStage::unfullscreen signal
  * @activate: handler for the #ClutterStage::activate signal
- * @deactivate: handler for the #ClutterStage::deactive signal
+ * @deactivate: handler for the #ClutterStage::deactivate signal
  * @delete_event: handler for the #ClutterStage::delete-event signal
  *
  * The #ClutterStageClass structure contains only private data
@@ -258,6 +258,10 @@ void                  clutter_stage_set_no_clear_hint (ClutterStage *stage,
                                                        gboolean      no_clear);
 gboolean              clutter_stage_get_no_clear_hint (ClutterStage *stage);
 
+void                  clutter_stage_set_accept_focus  (ClutterStage *stage,
+                                                       gboolean      accept_focus);
+gboolean              clutter_stage_get_accept_focus  (ClutterStage *stage);
+
 /* Commodity macro, for mallum only */
 #define clutter_stage_add(stage,actor)                  G_STMT_START {  \
   if (CLUTTER_IS_STAGE ((stage)) && CLUTTER_IS_ACTOR ((actor)))         \
