@@ -372,7 +372,7 @@ _cogl_framebuffer_clear4f (CoglFramebuffer *framebuffer,
    * batches from the journal. It is reset here to increase the
    * chances of getting the same colours for each frame during an
    * animation */
-  if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_RECTANGLES) &&
+  if (G_UNLIKELY (COGL_DEBUG_ENABLED (COGL_DEBUG_RECTANGLES)) &&
       buffers & COGL_BUFFER_BIT_COLOR)
     {
       _COGL_GET_CONTEXT (ctxt, NO_RETVAL);
@@ -596,6 +596,7 @@ _cogl_framebuffer_flush_dependency_journals (CoglFramebuffer *framebuffer)
   GList *l;
   for (l = framebuffer->deps; l; l = l->next)
     _cogl_framebuffer_flush_journal (l->data);
+  _cogl_framebuffer_remove_all_dependencies (framebuffer);
 }
 
 static inline void
@@ -1045,10 +1046,23 @@ _cogl_set_framebuffer_real (CoglFramebuffer *framebuffer)
 void
 cogl_set_framebuffer (CoglFramebuffer *framebuffer)
 {
+  CoglFramebuffer *current;
+
   g_return_if_fail (_cogl_is_framebuffer (framebuffer));
 
-  if (_cogl_get_framebuffer () != framebuffer)
-    _cogl_set_framebuffer_real (framebuffer);
+  current = _cogl_get_framebuffer ();
+  if (current != framebuffer)
+    {
+      /* XXX: eventually we want to remove this implicit journal flush
+       * so we can log into the journal beyond framebuffer changes to
+       * support batching scenes that depend on the results of
+       * mid-scene renders to textures. Current will be NULL when the
+       * framebuffer stack is first created so we need to guard
+       * against that here */
+      if (current)
+        _cogl_framebuffer_flush_journal (current);
+      _cogl_set_framebuffer_real (framebuffer);
+    }
 }
 
 /* XXX: deprecated API */
@@ -1081,8 +1095,11 @@ cogl_push_framebuffer (CoglFramebuffer *buffer)
   g_return_if_fail (_cogl_is_framebuffer (buffer));
   g_assert (ctx->framebuffer_stack);
 
+  /* Copy the top of the stack so that when we call cogl_set_framebuffer
+     it will still know what the old framebuffer was */
   ctx->framebuffer_stack =
-    g_slist_prepend (ctx->framebuffer_stack, COGL_INVALID_HANDLE);
+    g_slist_prepend (ctx->framebuffer_stack,
+                     cogl_object_ref (_cogl_get_framebuffer ()));
 
   cogl_set_framebuffer (buffer);
 }
@@ -1108,7 +1125,14 @@ cogl_pop_framebuffer (void)
   to_pop = ctx->framebuffer_stack->data;
   to_restore = ctx->framebuffer_stack->next->data;
 
-  cogl_flush ();
+  if (to_pop != to_restore)
+    {
+      /* XXX: eventually we want to remove this implicit journal flush
+       * so we can log into the journal beyond framebuffer changes to
+       * support batching scenes that depend on the results of
+       * mid-scene renders to textures. */
+      _cogl_framebuffer_flush_journal (to_pop);
+    }
 
   cogl_object_unref (to_pop);
   ctx->framebuffer_stack =
@@ -1237,7 +1261,7 @@ _cogl_framebuffer_try_fast_read_pixel (CoglFramebuffer *framebuffer,
 {
   gboolean found_intersection;
 
-  if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_DISABLE_FAST_READ_PIXEL))
+  if (G_UNLIKELY (COGL_DEBUG_ENABLED (COGL_DEBUG_DISABLE_FAST_READ_PIXEL)))
     return FALSE;
 
   if (source != COGL_READ_PIXELS_COLOR_BUFFER)
