@@ -707,9 +707,10 @@ static void
 clutter_stage_x11_dispose (GObject *gobject)
 {
   ClutterEventTranslator *translator = CLUTTER_EVENT_TRANSLATOR (gobject);
-  ClutterBackendX11 *backend_x11 = CLUTTER_STAGE_X11 (gobject)->backend;
+  ClutterBackendX11 *backend = CLUTTER_STAGE_X11 (gobject)->backend;
 
-  _clutter_backend_x11_remove_event_translator (backend_x11, translator);
+  _clutter_backend_remove_event_translator (CLUTTER_BACKEND (backend),
+                                            translator);
 
   G_OBJECT_CLASS (clutter_stage_x11_parent_class)->dispose (gobject);
 }
@@ -1082,6 +1083,13 @@ clutter_stage_x11_translate_event (ClutterEventTranslator *translator,
         }
       break;
 
+    case MappingNotify:
+      CLUTTER_NOTE (EVENT, "Refresh keyboard mapping");
+      XRefreshKeyboardMapping (&xevent->xmapping);
+      backend_x11->keymap_serial += 1;
+      res = CLUTTER_TRANSLATE_REMOVE;
+      break;
+
     default:
       res = CLUTTER_TRANSLATE_CONTINUE;
       break;
@@ -1097,7 +1105,7 @@ clutter_event_translator_iface_init (ClutterEventTranslatorIface *iface)
 }
 
 /**
- * clutter_x11_get_stage_window:
+ * clutter_x11_get_stage_window: (skip)
  * @stage: a #ClutterStage
  *
  * Gets the stages X Window.
@@ -1148,7 +1156,7 @@ clutter_x11_get_stage_from_window (Window win)
 }
 
 /**
- * clutter_x11_get_stage_visual:
+ * clutter_x11_get_stage_visual: (skip)
  * @stage: a #ClutterStage
  *
  * Returns an XVisualInfo suitable for creating a foreign window for the given
@@ -1333,7 +1341,13 @@ _clutter_stage_x11_destroy_window_untrapped (ClutterStageX11 *stage_x11)
   Window xwin = stage_x11->xwin;
 
   if (clutter_stages_by_xid != NULL)
-    g_hash_table_remove (clutter_stages_by_xid, GINT_TO_POINTER (xwin));
+    {
+      CLUTTER_NOTE (BACKEND, "Removing X11 stage 0x%x [%p]",
+                    (unsigned int) xwin,
+                    stage_x11);
+
+      g_hash_table_remove (clutter_stages_by_xid, GINT_TO_POINTER (xwin));
+    }
 
   if (!stage_x11->is_foreign_xwin && xwin != None)
     {
